@@ -4,13 +4,13 @@ use strict; use warnings;
 
 # Initialize our version
 use vars qw( $VERSION );
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 # auto-export the only sub we have
 require Exporter;
 use vars qw( @ISA @EXPORT_OK );
 @ISA = qw(Exporter);
-@EXPORT_OK = qw( poeloop2load loop2realversion beautify_times );
+@EXPORT_OK = qw( poeloop2load loop2realversion beautify_times knownloops );
 
 # returns the proper "load" stuff for a specific loop
 sub poeloop2load {
@@ -115,6 +115,11 @@ sub beautify_times {
 	return $data;
 }
 
+# returns a list of "known" POE loops
+sub knownloops {
+	return [ qw( Event_Lib EV Glib Prima Gtk Wx Kqueue Tk Select IO_Poll ) ];
+}
+
 1;
 __END__
 =head1 NAME
@@ -135,6 +140,10 @@ This package exports those subs via @EXPORT_OK:
 
 =over 4
 
+=item knownloops()
+
+Returns an arrayref of the "known" POE loops as of this version of the Benchmarker
+
 =item poeloop2load()
 
 Returns the "parent" class to load for a specific loop. An example is:
